@@ -24,19 +24,24 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #
 ##############################################################################
-import xml_marshaller
+
+import httplib
+import logging
+import os
 import unittest
 import urlparse
-import httplib
+
 import slapos.slap
-import os
+import xml_marshaller
+
 
 class UndefinedYetException(Exception):
   """To catch exceptions which are not yet defined"""
 
+
 class SlapMixin(unittest.TestCase):
   """
-  Usefull methods for slap tests
+  Useful methods for slap tests
   """
   def setUp(self):
     self._server_url = os.environ.get('TEST_SLAP_SERVER_URL', None)
@@ -54,13 +59,13 @@ class SlapMixin(unittest.TestCase):
 
   def _patchHttplib(self):
     """Overrides httplib"""
-    import mock.httplib
+    import slapmock.httplib
 
-    self.saved_httplib = dict()
+    self.saved_httplib = {}
 
-    for fake in vars(mock.httplib):
+    for fake in vars(slapmock.httplib):
       self.saved_httplib[fake] = getattr(httplib, fake, None)
-      setattr(httplib, fake, getattr(mock.httplib, fake))
+      setattr(httplib, fake, getattr(slapmock.httplib, fake))
 
   def _unpatchHttplib(self):
     """Restores httplib overriding"""
@@ -74,7 +79,7 @@ class SlapMixin(unittest.TestCase):
     Returns the computer id used by the test
     """
     return self.id()
-    return os.environ.get('TEST_SLAP_COMPUTER_ID', self.id())
+
 
 class TestSlap(SlapMixin):
   """
@@ -88,10 +93,8 @@ class TestSlap(SlapMixin):
     """
     slap_instance = slapos.slap.slap()
     slap_instance.initializeConnection(self.server_url)
-    self.assertTrue(
-        slap_instance._connection_helper.host in self.server_url)
-    self.assertTrue(
-        slap_instance._connection_helper.path in self.server_url)
+    self.assertIn(slap_instance._connection_helper.host, self.server_url)
+    self.assertIn(slap_instance._connection_helper.path, self.server_url)
 
   def test_slap_initialisation_wrong_url(self):
     """
@@ -101,7 +104,8 @@ class TestSlap(SlapMixin):
     server_url = 'https://user:pass@server/path/path?parameter=notAcceptable'
     slap_instance = slapos.slap.slap()
     self.assertRaises(AttributeError,
-                      slap_instance.initializeConnection, server_url)
+                      slap_instance.initializeConnection,
+                      server_url)
 
   def test_registerComputer_with_new_guid(self):
     """
@@ -112,7 +116,7 @@ class TestSlap(SlapMixin):
     self.slap = slapos.slap.slap()
     self.slap.initializeConnection(self.server_url)
     computer = self.slap.registerComputer(computer_guid)
-    self.assertTrue(isinstance(computer, slapos.slap.Computer))
+    self.assertIsInstance(computer, slapos.slap.Computer)
 
   def test_registerComputer_with_existing_guid(self):
     """
@@ -123,10 +127,10 @@ class TestSlap(SlapMixin):
     self.slap = slapos.slap.slap()
     self.slap.initializeConnection(self.server_url)
     computer = self.slap.registerComputer(computer_guid)
-    self.assertTrue(isinstance(computer, slapos.slap.Computer))
+    self.assertIsInstance(computer, slapos.slap.Computer)
 
     computer2 = self.slap.registerComputer(computer_guid)
-    self.assertTrue(isinstance(computer2, slapos.slap.Computer))
+    self.assertIsInstance(computer2, slapos.slap.Computer)
 
   # XXX: There is naming conflict in slap library.
   # SoftwareRelease is currently used as suboject of Slap transmission object
@@ -139,8 +143,7 @@ class TestSlap(SlapMixin):
     self.slap = slapos.slap.slap()
     self.slap.initializeConnection(self.server_url)
     software_release = self.slap.registerSoftwareRelease(software_release_uri)
-    self.assertTrue(isinstance(software_release,
-        slapos.slap.SoftwareRelease))
+    self.assertIsInstance(software_release, slapos.slap.SoftwareRelease)
 
   def test_registerSoftwareRelease_with_existing_uri(self):
     """
@@ -151,12 +154,10 @@ class TestSlap(SlapMixin):
     self.slap = slapos.slap.slap()
     self.slap.initializeConnection(self.server_url)
     software_release = self.slap.registerSoftwareRelease(software_release_uri)
-    self.assertTrue(isinstance(software_release,
-        slapos.slap.SoftwareRelease))
+    self.assertIsInstance(software_release, slapos.slap.SoftwareRelease)
 
     software_release2 = self.slap.registerSoftwareRelease(software_release_uri)
-    self.assertTrue(isinstance(software_release2,
-        slapos.slap.SoftwareRelease))
+    self.assertIsInstance(software_release2, slapos.slap.SoftwareRelease)
 
   def test_registerComputerPartition_new_partition_id_known_computer_guid(self):
     """
@@ -171,9 +172,9 @@ class TestSlap(SlapMixin):
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(parsed_url.query)
-      if parsed_url.path == 'registerComputerPartition' and \
-         parsed_qs['computer_reference'][0] == computer_guid and \
-         parsed_qs['computer_partition_reference'][0] == partition_id:
+      if (parsed_url.path == 'registerComputerPartition'
+              and parsed_qs['computer_reference'][0] == computer_guid
+              and parsed_qs['computer_partition_reference'][0] == partition_id):
         partition = slapos.slap.ComputerPartition(
             computer_guid, partition_id)
         return (200, {}, xml_marshaller.xml_marshaller.dumps(partition))
@@ -182,7 +183,7 @@ class TestSlap(SlapMixin):
     httplib.HTTPConnection._callback = server_response
 
     partition = self.slap.registerComputerPartition(computer_guid, partition_id)
-    self.assertTrue(isinstance(partition, slapos.slap.ComputerPartition))
+    self.assertIsInstance(partition, slapos.slap.ComputerPartition)
 
   def test_registerComputerPartition_existing_partition_id_known_computer_guid(self):
     """
@@ -191,8 +192,8 @@ class TestSlap(SlapMixin):
     """
     self.test_registerComputerPartition_new_partition_id_known_computer_guid()
     partition = self.slap.registerComputerPartition(self._getTestComputerId(),
-        self.partition_id)
-    self.assertTrue(isinstance(partition, slapos.slap.ComputerPartition))
+                                                    self.partition_id)
+    self.assertIsInstance(partition, slapos.slap.ComputerPartition)
 
   def test_registerComputerPartition_unknown_computer_guid(self):
     """
@@ -206,9 +207,9 @@ class TestSlap(SlapMixin):
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(parsed_url.query)
-      if parsed_url.path == 'registerComputerPartition' and \
-         parsed_qs['computer_reference'][0] == computer_guid and \
-         parsed_qs['computer_partition_reference'][0] == partition_id:
+      if (parsed_url.path == 'registerComputerPartition'
+              and parsed_qs['computer_reference'][0] == computer_guid
+              and parsed_qs['computer_partition_reference'][0] == partition_id):
         slapos.slap.ComputerPartition(computer_guid, partition_id)
         return (404, {}, '')
       else:
@@ -216,7 +217,8 @@ class TestSlap(SlapMixin):
     httplib.HTTPConnection._callback = server_response
 
     self.assertRaises(slapos.slap.NotFoundError,
-        self.slap.registerComputerPartition, computer_guid, partition_id)
+                      self.slap.registerComputerPartition,
+                      computer_guid, partition_id)
 
   def test_getFullComputerInformation_empty_computer_guid(self):
     """
@@ -228,11 +230,12 @@ class TestSlap(SlapMixin):
     def server_response(self_httpconnection, path, method, body, header):
       # Shouldn't even be called
       self.assertFalse(True)
+
     httplib.HTTPConnection._callback = server_response
 
-    self.assertRaises(
-        slapos.slap.NotFoundError,
-        self.slap._connection_helper.getFullComputerInformation, None)
+    self.assertRaises(slapos.slap.NotFoundError,
+                      self.slap._connection_helper.getFullComputerInformation,
+                      None)
 
   def test_registerComputerPartition_empty_computer_guid(self):
     """
@@ -244,11 +247,12 @@ class TestSlap(SlapMixin):
     def server_response(self_httpconnection, path, method, body, header):
       # Shouldn't even be called
       self.assertFalse(True)
+
     httplib.HTTPConnection._callback = server_response
 
-    self.assertRaises(
-        slapos.slap.NotFoundError,
-        self.slap.registerComputerPartition, None, 'PARTITION_01')
+    self.assertRaises(slapos.slap.NotFoundError,
+                      self.slap.registerComputerPartition,
+                      None, 'PARTITION_01')
 
   def test_registerComputerPartition_empty_computer_partition_id(self):
     """
@@ -260,12 +264,12 @@ class TestSlap(SlapMixin):
     def server_response(self_httpconnection, path, method, body, header):
       # Shouldn't even be called
       self.assertFalse(True)
+
     httplib.HTTPConnection._callback = server_response
 
-    self.assertRaises(
-        slapos.slap.NotFoundError,
-        self.slap.registerComputerPartition,
-        self._getTestComputerId(), None)
+    self.assertRaises(slapos.slap.NotFoundError,
+                      self.slap.registerComputerPartition,
+                      self._getTestComputerId(), None)
 
   def test_registerComputerPartition_empty_computer_guid_empty_computer_partition_id(self):
     """
@@ -277,12 +281,81 @@ class TestSlap(SlapMixin):
     def server_response(self_httpconnection, path, method, body, header):
       # Shouldn't even be called
       self.assertFalse(True)
+
     httplib.HTTPConnection._callback = server_response
 
+    self.assertRaises(slapos.slap.NotFoundError,
+                      self.slap.registerComputerPartition,
+                      None, None)
+
+
+  def test_getSoftwareReleaseListFromSoftwareProduct_software_product_reference(self):
+    """
+    Check that slap.getSoftwareReleaseListFromSoftwareProduct calls
+    "/getSoftwareReleaseListFromSoftwareProduct" URL with correct parameters,
+    with software_product_reference parameter being specified.
+    """
+    self.slap.initializeConnection(self.server_url)
+    software_product_reference = 'random_reference'
+    software_release_url_list = ['1', '2']
+
+    def server_response(self_httpconnection, path, method, body, header):
+      parsed_url = urlparse.urlparse(path.lstrip('/'))
+      parsed_qs = urlparse.parse_qs(parsed_url.query)
+      if parsed_url.path == 'getSoftwareReleaseListFromSoftwareProduct' \
+         and parsed_qs == {'software_product_reference': [software_product_reference]}:
+        return (200, {}, xml_marshaller.xml_marshaller.dumps(software_release_url_list))
+    httplib.HTTPConnection._callback = server_response
+
+    self.assertEqual(
+      self.slap.getSoftwareReleaseListFromSoftwareProduct(
+          software_product_reference=software_product_reference),
+      software_release_url_list
+    )
+
+  def test_getSoftwareReleaseListFromSoftwareProduct_software_release_url(self):
+    """
+    Check that slap.getSoftwareReleaseListFromSoftwareProduct calls
+    "/getSoftwareReleaseListFromSoftwareProduct" URL with correct parameters,
+    with software_release_url parameter being specified.
+    """
+    self.slap.initializeConnection(self.server_url)
+    software_release_url = 'random_url'
+    software_release_url_list = ['1', '2']
+
+    def server_response(self_httpconnection, path, method, body, header):
+      parsed_url = urlparse.urlparse(path.lstrip('/'))
+      parsed_qs = urlparse.parse_qs(parsed_url.query)
+      if parsed_url.path == 'getSoftwareReleaseListFromSoftwareProduct' \
+         and parsed_qs == {'software_release_url': [software_release_url]}:
+        return (200, {}, xml_marshaller.xml_marshaller.dumps(software_release_url_list))
+    httplib.HTTPConnection._callback = server_response
+
+    self.assertEqual(
+      self.slap.getSoftwareReleaseListFromSoftwareProduct(
+          software_release_url=software_release_url),
+      software_release_url_list
+    )
+
+  def test_getSoftwareReleaseListFromSoftwareProduct_too_many_parameters(self):
+    """
+    Check that slap.getSoftwareReleaseListFromSoftwareProduct raises if
+    both parameters are set.
+    """
     self.assertRaises(
-        slapos.slap.NotFoundError,
-        self.slap.registerComputerPartition,
-        None, None)
+      AttributeError,
+      self.slap.getSoftwareReleaseListFromSoftwareProduct, 'foo', 'bar'
+    )
+
+  def test_getSoftwareReleaseListFromSoftwareProduct_no_parameter(self):
+    """
+    Check that slap.getSoftwareReleaseListFromSoftwareProduct raises if
+    both parameters are either not set or None.
+    """
+    self.assertRaises(
+      AttributeError,
+      self.slap.getSoftwareReleaseListFromSoftwareProduct
+    )
 
 class TestComputer(SlapMixin):
   """
@@ -301,15 +374,15 @@ class TestComputer(SlapMixin):
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(parsed_url.query)
-      if parsed_url.path == 'registerComputerPartition' and \
-         'computer_reference' in parsed_qs and \
-         'computer_partition_reference' in parsed_qs:
+      if (parsed_url.path == 'registerComputerPartition'
+              and 'computer_reference' in parsed_qs
+              and 'computer_partition_reference' in parsed_qs):
         slap_partition = slapos.slap.ComputerPartition(
-          parsed_qs['computer_reference'][0],
-          parsed_qs['computer_partition_reference'][0])
+            parsed_qs['computer_reference'][0],
+            parsed_qs['computer_partition_reference'][0])
         return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_partition))
-      elif parsed_url.path == 'getFullComputerInformation' and \
-         'computer_id' in parsed_qs:
+      elif (parsed_url.path == 'getFullComputerInformation'
+              and 'computer_id' in parsed_qs):
         slap_computer = slapos.slap.Computer(parsed_qs['computer_id'][0])
         slap_computer._software_release_list = []
         slap_computer._computer_partition_list = []
@@ -333,12 +406,12 @@ class TestComputer(SlapMixin):
     def server_response(self_httpconnection, path, method, body, header):
       # Shouldn't even be called
       self.assertFalse(True)
+
     httplib.HTTPConnection._callback = server_response
 
     computer = self.slap.registerComputer(None)
-    self.assertRaises(
-        slapos.slap.NotFoundError,
-        getattr(computer, computer_method))
+    self.assertRaises(slapos.slap.NotFoundError,
+                      getattr(computer, computer_method))
 
   def test_computer_getComputerPartitionList_empty_computer_guid(self):
     """
@@ -365,7 +438,7 @@ class TestComputer(SlapMixin):
     self.slap.initializeConnection(self.server_url)
     self.computer = self.slap.registerComputer(self.computer_guid)
     self.partition = self.slap.registerComputerPartition(self.computer_guid,
-        partition_id)
+                                                         partition_id)
     self.assertEqual(self.computer.getComputerPartitionList(), [])
 
   @unittest.skip("Not implemented")
@@ -375,7 +448,6 @@ class TestComputer(SlapMixin):
     (not defined yet) XML raises (not defined yet) exception
     """
 
-
     self.computer_guid = self._getTestComputerId()
     self.slap = slapos.slap.slap()
     self.slap.initializeConnection(self.server_url)
@@ -383,8 +455,9 @@ class TestComputer(SlapMixin):
     non_dtd_xml = """<xml>
 <non-dtd-parameter name="xerxes">value<non-dtd-parameter name="xerxes">
 </xml>"""
-    self.assertRaises(UndefinedYetException, self.computer.reportUsage,
-        non_dtd_xml)
+    self.assertRaises(UndefinedYetException,
+                      self.computer.reportUsage,
+                      non_dtd_xml)
 
   @unittest.skip("Not implemented")
   def test_computer_reportUsage_valid_xml_invalid_partition_raises(self):
@@ -399,17 +472,20 @@ class TestComputer(SlapMixin):
     self.slap.initializeConnection(self.server_url)
     self.computer = self.slap.registerComputer(self.computer_guid)
     self.partition = self.slap.registerComputerPartition(self.computer_guid,
-        partition_id)
+                                                         partition_id)
     # XXX: As DTD is not defined currently proper XML is not known
     bad_partition_dtd_xml = """<xml>
 <computer-partition id='ANOTHER_PARTITION>96.5% CPU</computer-partition>
 </xml>"""
-    self.assertRaises(UndefinedYetException, self.computer.reportUsage,
-        bad_partition_dtd_xml)
+    self.assertRaises(UndefinedYetException,
+                      self.computer.reportUsage,
+                      bad_partition_dtd_xml)
+
 
 class RequestWasCalled(Exception):
   pass
 
+
 class TestComputerPartition(SlapMixin):
   """
   Tests slapos.slap.slap.ComputerPartition class functionality
@@ -417,65 +493,69 @@ class TestComputerPartition(SlapMixin):
 
   def test_request_sends_request(self):
     partition_id = 'PARTITION_01'
+
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(parsed_url.query)
-      if parsed_url.path == 'registerComputerPartition' and \
-         'computer_reference' in parsed_qs and \
-         'computer_partition_reference' in parsed_qs:
+      if (parsed_url.path == 'registerComputerPartition'
+              and 'computer_reference' in parsed_qs
+              and 'computer_partition_reference' in parsed_qs):
         slap_partition = slapos.slap.ComputerPartition(
-          parsed_qs['computer_reference'][0],
-          parsed_qs['computer_partition_reference'][0])
+            parsed_qs['computer_reference'][0],
+            parsed_qs['computer_partition_reference'][0])
         return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_partition))
-      elif parsed_url.path == 'getComputerInformation' and \
-         'computer_id' in parsed_qs:
+      elif (parsed_url.path == 'getComputerInformation'
+              and 'computer_id' in parsed_qs):
         slap_computer = slapos.slap.Computer(parsed_qs['computer_id'][0])
         slap_computer._software_release_list = []
         slap_partition = slapos.slap.ComputerPartition(
-          parsed_qs['computer_id'][0],
-          partition_id)
+            parsed_qs['computer_id'][0],
+            partition_id)
         slap_computer._computer_partition_list = [slap_partition]
         return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_computer))
       elif parsed_url.path == 'requestComputerPartition':
         raise RequestWasCalled
       else:
         return (404, {}, '')
+
     httplib.HTTPConnection._callback = server_response
     self.computer_guid = self._getTestComputerId()
     self.slap = slapos.slap.slap()
     self.slap.initializeConnection(self.server_url)
     computer_partition = self.slap.registerComputerPartition(
         self.computer_guid, partition_id)
-    self.assertRaises(RequestWasCalled, computer_partition.request,
-        'http://server/new/' + self._getTestComputerId(),
-        'software_type',
-        'myref')
+    self.assertRaises(RequestWasCalled,
+                      computer_partition.request,
+                      'http://server/new/' + self._getTestComputerId(),
+                      'software_type', 'myref')
 
   def test_request_not_raises(self):
     partition_id = 'PARTITION_01'
+
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(parsed_url.query)
-      if parsed_url.path == 'registerComputerPartition' and \
-         'computer_reference' in parsed_qs and \
-         'computer_partition_reference' in parsed_qs:
+      if (parsed_url.path == 'registerComputerPartition'
+              and 'computer_reference' in parsed_qs
+              and 'computer_partition_reference' in parsed_qs):
         slap_partition = slapos.slap.ComputerPartition(
-          parsed_qs['computer_reference'][0],
-          parsed_qs['computer_partition_reference'][0])
+            parsed_qs['computer_reference'][0],
+            parsed_qs['computer_partition_reference'][0])
         return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_partition))
-      elif parsed_url.path == 'getComputerInformation' and \
-         'computer_id' in parsed_qs:
+      elif (parsed_url.path == 'getComputerInformation'
+              and 'computer_id' in parsed_qs):
         slap_computer = slapos.slap.Computer(parsed_qs['computer_id'][0])
         slap_computer._software_release_list = []
         slap_partition = slapos.slap.ComputerPartition(
-          parsed_qs['computer_id'][0],
-          partition_id)
+            parsed_qs['computer_id'][0],
+            partition_id)
         slap_computer._computer_partition_list = [slap_partition]
         return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_computer))
       elif parsed_url.path == 'requestComputerPartition':
         return (408, {}, '')
       else:
         return (404, {}, '')
+
     httplib.HTTPConnection._callback = server_response
     self.computer_guid = self._getTestComputerId()
     self.slap = slapos.slap.slap()
@@ -486,34 +566,35 @@ class TestComputerPartition(SlapMixin):
         'http://server/new/' + self._getTestComputerId(),
         'software_type',
         'myref')
-    self.assertTrue(isinstance(requested_partition,
-        slapos.slap.ComputerPartition))
+    self.assertIsInstance(requested_partition, slapos.slap.ComputerPartition)
 
   def test_request_raises_later(self):
     partition_id = 'PARTITION_01'
+
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(parsed_url.query)
-      if parsed_url.path == 'registerComputerPartition' and \
-         'computer_reference' in parsed_qs and \
-         'computer_partition_reference' in parsed_qs:
+      if (parsed_url.path == 'registerComputerPartition' and
+              'computer_reference' in parsed_qs and
+              'computer_partition_reference' in parsed_qs):
         slap_partition = slapos.slap.ComputerPartition(
-          parsed_qs['computer_reference'][0],
-          parsed_qs['computer_partition_reference'][0])
+            parsed_qs['computer_reference'][0],
+            parsed_qs['computer_partition_reference'][0])
         return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_partition))
-      elif parsed_url.path == 'getComputerInformation' and \
-         'computer_id' in parsed_qs:
+      elif (parsed_url.path == 'getComputerInformation'
+              and 'computer_id' in parsed_qs):
         slap_computer = slapos.slap.Computer(parsed_qs['computer_id'][0])
         slap_computer._software_release_list = []
         slap_partition = slapos.slap.ComputerPartition(
-          parsed_qs['computer_id'][0],
-          partition_id)
+            parsed_qs['computer_id'][0],
+            partition_id)
         slap_computer._computer_partition_list = [slap_partition]
         return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_computer))
       elif parsed_url.path == 'requestComputerPartition':
         return (408, {}, '')
       else:
         return (404, {}, '')
+
     httplib.HTTPConnection._callback = server_response
     self.computer_guid = self._getTestComputerId()
     self.slap = slapos.slap.slap()
@@ -524,8 +605,7 @@ class TestComputerPartition(SlapMixin):
         'http://server/new/' + self._getTestComputerId(),
         'software_type',
         'myref')
-    self.assertTrue(isinstance(requested_partition,
-        slapos.slap.ComputerPartition))
+    self.assertIsInstance(requested_partition, slapos.slap.ComputerPartition)
     # as request method does not raise, accessing data raises
     self.assertRaises(slapos.slap.ResourceNotReady,
                       requested_partition.getId)
@@ -534,33 +614,34 @@ class TestComputerPartition(SlapMixin):
     partition_id = 'PARTITION_01'
     requested_partition_id = 'PARTITION_02'
     computer_guid = self._getTestComputerId()
+
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(parsed_url.query)
-      if parsed_url.path == 'registerComputerPartition' and \
-         'computer_reference' in parsed_qs and \
-         'computer_partition_reference' in parsed_qs:
+      if (parsed_url.path == 'registerComputerPartition' and
+              'computer_reference' in parsed_qs and
+              'computer_partition_reference' in parsed_qs):
         slap_partition = slapos.slap.ComputerPartition(
-          parsed_qs['computer_reference'][0],
-          parsed_qs['computer_partition_reference'][0])
+            parsed_qs['computer_reference'][0],
+            parsed_qs['computer_partition_reference'][0])
         return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_partition))
-      elif parsed_url.path == 'getComputerInformation' and \
-         'computer_id' in parsed_qs:
+      elif (parsed_url.path == 'getComputerInformation' and 'computer_id' in parsed_qs):
         slap_computer = slapos.slap.Computer(parsed_qs['computer_id'][0])
         slap_computer._software_release_list = []
         slap_partition = slapos.slap.ComputerPartition(
-          parsed_qs['computer_id'][0],
-          partition_id)
+            parsed_qs['computer_id'][0],
+            partition_id)
         slap_computer._computer_partition_list = [slap_partition]
         return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_computer))
       elif parsed_url.path == 'requestComputerPartition':
         from slapos.slap.slap import SoftwareInstance
         slap_partition = SoftwareInstance(
-          slap_computer_id=computer_guid,
-          slap_computer_partition_id=requested_partition_id)
+            slap_computer_id=computer_guid,
+            slap_computer_partition_id=requested_partition_id)
         return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_partition))
       else:
         return (404, {}, '')
+
     httplib.HTTPConnection._callback = server_response
     self.slap = slapos.slap.slap()
     self.slap.initializeConnection(self.server_url)
@@ -570,8 +651,7 @@ class TestComputerPartition(SlapMixin):
         'http://server/new/' + self._getTestComputerId(),
         'software_type',
         'myref')
-    self.assertTrue(isinstance(requested_partition,
-        slapos.slap.ComputerPartition))
+    self.assertIsInstance(requested_partition, slapos.slap.ComputerPartition)
     # as request method does not raise, accessing data in case when
     # request was done works correctly
     self.assertEqual(requested_partition_id, requested_partition.getId())
@@ -589,9 +669,9 @@ class TestComputerPartition(SlapMixin):
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(parsed_url.query)
-      if parsed_url.path == 'registerComputerPartition' and \
-         parsed_qs['computer_reference'][0] == computer_guid and \
-         parsed_qs['computer_partition_reference'][0] == partition_id:
+      if (parsed_url.path == 'registerComputerPartition' and
+              parsed_qs['computer_reference'][0] == computer_guid and
+              parsed_qs['computer_partition_reference'][0] == partition_id):
         partition = slapos.slap.ComputerPartition(
             computer_guid, partition_id)
         return (200, {}, xml_marshaller.xml_marshaller.dumps(partition))
@@ -601,8 +681,8 @@ class TestComputerPartition(SlapMixin):
 
     computer_partition = self.slap.registerComputerPartition(
         computer_guid, partition_id)
-    method = getattr(computer_partition, state)
-    self.assertRaises(slapos.slap.NotFoundError, method)
+    self.assertRaises(slapos.slap.NotFoundError,
+                      getattr(computer_partition, state))
 
   def test_available_new_ComputerPartition_raises(self):
     """
@@ -644,9 +724,9 @@ class TestComputerPartition(SlapMixin):
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(parsed_url.query)
-      if parsed_url.path == 'registerComputerPartition' and \
-         parsed_qs['computer_reference'][0] == computer_guid and \
-         parsed_qs['computer_partition_reference'][0] == partition_id:
+      if (parsed_url.path == 'registerComputerPartition' and
+              parsed_qs['computer_reference'][0] == computer_guid and
+              parsed_qs['computer_partition_reference'][0] == partition_id):
         partition = slapos.slap.ComputerPartition(
             computer_guid, partition_id)
         return (200, {}, xml_marshaller.xml_marshaller.dumps(partition))
@@ -655,9 +735,9 @@ class TestComputerPartition(SlapMixin):
         # XXX: why do we have computer_id and not computer_reference?
         # XXX: why do we have computer_partition_id and not
         # computer_partition_reference?
-        if parsed_qs_body['computer_id'][0] == computer_guid and \
-           parsed_qs_body['computer_partition_id'][0] == partition_id and \
-           parsed_qs_body['error_log'][0] == 'some error':
+        if (parsed_qs_body['computer_id'][0] == computer_guid and
+                parsed_qs_body['computer_partition_id'][0] == partition_id and
+                parsed_qs_body['error_log'][0] == 'some error'):
           return (200, {}, '')
 
       return (404, {}, '')
@@ -668,6 +748,7 @@ class TestComputerPartition(SlapMixin):
     # XXX: Interface does not define return value
     computer_partition.error('some error')
 
+
 class TestSoftwareRelease(SlapMixin):
   """
   Tests slap.SoftwareRelease class functionality
@@ -712,18 +793,20 @@ class TestSoftwareRelease(SlapMixin):
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(body)
-      if parsed_url.path == 'softwareReleaseError' and \
-         parsed_qs['computer_id'][0] == computer_guid and \
-         parsed_qs['url'][0] == software_release_uri and \
-         parsed_qs['error_log'][0] == 'some error':
+      if (parsed_url.path == 'softwareReleaseError' and
+              parsed_qs['computer_id'][0] == computer_guid and
+              parsed_qs['url'][0] == software_release_uri and
+              parsed_qs['error_log'][0] == 'some error'):
         return (200, {}, '')
       return (404, {}, '')
+
     httplib.HTTPConnection._callback = server_response
 
     software_release = self.slap.registerSoftwareRelease(software_release_uri)
     software_release._computer_guid = computer_guid
     software_release.error('some error')
 
+
 class TestOpenOrder(SlapMixin):
   def test_request_sends_request(self):
     software_release_uri = 'http://server/new/' + self._getTestComputerId()
@@ -731,12 +814,16 @@ class TestOpenOrder(SlapMixin):
     self.slap.initializeConnection(self.server_url)
     # XXX: Interface lack registerOpenOrder method declaration
     open_order = self.slap.registerOpenOrder()
+
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       if parsed_url.path == 'requestComputerPartition':
         raise RequestWasCalled
+
     httplib.HTTPConnection._callback = server_response
-    self.assertRaises(RequestWasCalled, open_order.request, software_release_uri, 'myrefe')
+    self.assertRaises(RequestWasCalled,
+                      open_order.request,
+                      software_release_uri, 'myrefe')
 
   def test_request_not_raises(self):
     software_release_uri = 'http://server/new/' + self._getTestComputerId()
@@ -745,8 +832,7 @@ class TestOpenOrder(SlapMixin):
     # XXX: Interface lack registerOpenOrder method declaration
     open_order = self.slap.registerOpenOrder()
     computer_partition = open_order.request(software_release_uri, 'myrefe')
-    self.assertTrue(isinstance(computer_partition,
-        slapos.slap.ComputerPartition))
+    self.assertIsInstance(computer_partition, slapos.slap.ComputerPartition)
 
   def test_request_raises_later(self):
     software_release_uri = 'http://server/new/' + self._getTestComputerId()
@@ -754,14 +840,16 @@ class TestOpenOrder(SlapMixin):
     self.slap.initializeConnection(self.server_url)
     # XXX: Interface lack registerOpenOrder method declaration
     open_order = self.slap.registerOpenOrder()
+
     def server_response(self, path, method, body, header):
       return (408, {}, '')
+
     httplib.HTTPConnection._callback = server_response
     computer_partition = open_order.request(software_release_uri, 'myrefe')
-    self.assertTrue(isinstance(computer_partition,
-        slapos.slap.ComputerPartition))
+    self.assertIsInstance(computer_partition, slapos.slap.ComputerPartition)
 
-    self.assertRaises(slapos.slap.ResourceNotReady, computer_partition.getId)
+    self.assertRaises(slapos.slap.ResourceNotReady,
+                      computer_partition.getId)
 
   def test_request_fullfilled_work(self):
     software_release_uri = 'http://server/new/' + self._getTestComputerId()
@@ -771,18 +859,129 @@ class TestOpenOrder(SlapMixin):
     open_order = self.slap.registerOpenOrder()
     computer_guid = self._getTestComputerId()
     requested_partition_id = 'PARTITION_01'
+
     def server_response(self, path, method, body, header):
       from slapos.slap.slap import SoftwareInstance
       slap_partition = SoftwareInstance(
-        slap_computer_id=computer_guid,
-        slap_computer_partition_id=requested_partition_id)
+          slap_computer_id=computer_guid,
+          slap_computer_partition_id=requested_partition_id)
       return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_partition))
+
     httplib.HTTPConnection._callback = server_response
 
     computer_partition = open_order.request(software_release_uri, 'myrefe')
-    self.assertTrue(isinstance(computer_partition,
-        slapos.slap.ComputerPartition))
-    self.assertTrue(requested_partition_id, computer_partition.getId())
+    self.assertIsInstance(computer_partition, slapos.slap.ComputerPartition)
+    self.assertEqual(requested_partition_id, computer_partition.getId())
+
+
+  def test_request_getConnectionParameter(self):
+    """ Backward compatibility API for slapproxy older them 1.0.1 """
+    software_release_uri = 'http://server/new/' + self._getTestComputerId()
+    self.slap = slapos.slap.slap()
+    self.slap.initializeConnection(self.server_url)
+    # XXX: Interface lack registerOpenOrder method declaration
+    open_order = self.slap.registerOpenOrder()
+    computer_guid = self._getTestComputerId()
+    requested_partition_id = 'PARTITION_01'
+
+    def server_response(self, path, method, body, header):
+      from slapos.slap.slap import SoftwareInstance
+      slap_partition = SoftwareInstance(
+          _connection_dict = {"url": 'URL_CONNECTION_PARAMETER'},
+          slap_computer_id=computer_guid,
+          slap_computer_partition_id=requested_partition_id)
+      return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_partition))
+
+    httplib.HTTPConnection._callback = server_response
+
+    computer_partition = open_order.request(software_release_uri, 'myrefe')
+    self.assertIsInstance(computer_partition, slapos.slap.ComputerPartition)
+    self.assertEqual(requested_partition_id, computer_partition.getId())
+    self.assertEqual("URL_CONNECTION_PARAMETER", 
+                     computer_partition.getConnectionParameter('url'))
+
+
+  def test_request_connection_dict_backward_compatibility(self):
+    """ Backward compatibility API for slapproxy older them 1.0.1 """
+    software_release_uri = 'http://server/new/' + self._getTestComputerId()
+    self.slap = slapos.slap.slap()
+    self.slap.initializeConnection(self.server_url)
+    # XXX: Interface lack registerOpenOrder method declaration
+    open_order = self.slap.registerOpenOrder()
+    computer_guid = self._getTestComputerId()
+    requested_partition_id = 'PARTITION_01'
+
+    def server_response(self, path, method, body, header):
+      from slapos.slap.slap import SoftwareInstance
+      slap_partition = SoftwareInstance(
+          connection_xml="""<?xml version='1.0' encoding='utf-8'?>
+<instance>
+  <parameter id="url">URL_CONNECTION_PARAMETER</parameter>
+</instance>""",
+          slap_computer_id=computer_guid,
+          slap_computer_partition_id=requested_partition_id)
+      return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_partition))
+
+    httplib.HTTPConnection._callback = server_response
+
+    computer_partition = open_order.request(software_release_uri, 'myrefe')
+    self.assertIsInstance(computer_partition, slapos.slap.ComputerPartition)
+    self.assertEqual(requested_partition_id, computer_partition.getId())
+    self.assertEqual("URL_CONNECTION_PARAMETER", 
+                     computer_partition.getConnectionParameter('url'))
+
+
+class TestSoftwareProductCollection(SlapMixin):
+  def setUp(self):
+    SlapMixin.setUp(self)
+    self.real_getSoftwareReleaseListFromSoftwareProduct =\
+        slapos.slap.slap.getSoftwareReleaseListFromSoftwareProduct
+
+    def fake_getSoftwareReleaseListFromSoftwareProduct(inside_self, software_product_reference):
+      return self.getSoftwareReleaseListFromSoftwareProduct_response
+    slapos.slap.slap.getSoftwareReleaseListFromSoftwareProduct =\
+        fake_getSoftwareReleaseListFromSoftwareProduct
+
+    self.product_collection = slapos.slap.SoftwareProductCollection(
+        logging.getLogger(), slapos.slap.slap())
+
+  def tearDown(self):
+    slapos.slap.slap.getSoftwareReleaseListFromSoftwareProduct =\
+        self.real_getSoftwareReleaseListFromSoftwareProduct
+
+  def test_get_product(self):
+    """
+    Test that the get method (aliased to __getattr__) returns the first element
+    of the list given by getSoftwareReleaseListFromSoftwareProduct (i.e the
+    best one).
+    """
+    self.getSoftwareReleaseListFromSoftwareProduct_response = ['0', '1', '2']
+    self.assertEqual(
+      self.product_collection.get('random_reference'),
+      self.getSoftwareReleaseListFromSoftwareProduct_response[0]
+    )
+
+  def test_get_product_empty_product(self):
+    """
+    Test that the get method (aliased to __getattr__) raises if no
+    Software Release is related to the Software Product, or if the
+    Software Product does not exist.
+    """
+    self.getSoftwareReleaseListFromSoftwareProduct_response = []
+    self.assertRaises(
+      AttributeError,
+      self.product_collection.get, 'random_reference',
+    )
+
+  def test_get_product_gettattr(self):
+    """
+    Test that __getattr__ method is bound to get() method.
+    """
+    self.getSoftwareReleaseListFromSoftwareProduct_response = []
+    self.assertEqual(
+      self.product_collection.__getattr__,
+      self.product_collection.get
+    )
 
 if __name__ == '__main__':
   print 'You can point to any SLAP server by setting TEST_SLAP_SERVER_URL '\
